package fr.imag.adele.maven.plugin.obr;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.apache.maven.artifact.manager.WagonConfigurationException;
import org.apache.maven.artifact.manager.WagonManager;
import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.logging.Log;
import org.apache.maven.settings.Proxy;
import org.apache.maven.settings.Settings;
import org.apache.maven.wagon.ConnectionException;
import org.apache.maven.wagon.ResourceDoesNotExistException;
import org.apache.maven.wagon.TransferFailedException;
import org.apache.maven.wagon.UnsupportedProtocolException;
import org.apache.maven.wagon.Wagon;
import org.apache.maven.wagon.authentication.AuthenticationException;
import org.apache.maven.wagon.authorization.AuthorizationException;
import org.apache.maven.wagon.observers.Debug;
import org.apache.maven.wagon.proxy.ProxyInfo;
import org.apache.maven.wagon.repository.Repository;
import org.codehaus.plexus.PlexusConstants;
import org.codehaus.plexus.PlexusContainer;
import org.codehaus.plexus.component.configurator.ComponentConfigurationException;
import org.codehaus.plexus.component.configurator.ComponentConfigurator;
import org.codehaus.plexus.component.repository.exception.ComponentLifecycleException;
import org.codehaus.plexus.component.repository.exception.ComponentLookupException;
import org.codehaus.plexus.configuration.PlexusConfiguration;
import org.codehaus.plexus.context.Context;
import org.codehaus.plexus.context.ContextException;


/**
 * this class is used to manage all connections by wagon 
 **/
public class RemoteFileManager {

    private Wagon m_wagon;
    private WagonManager m_wagonManager;
    private ArtifactRepository m_artifactRepository;
    private Settings m_settings;
    private Log m_log;
    
    private PlexusContainer container;
    /** Map( String, XmlPlexusConfiguration ) with the repository id and the wagon configuration */
    private Map serverConfigurationMap = new HashMap();
    
    public RemoteFileManager(ArtifactRepository ar, WagonManager wm, Settings settings,
                            Log log){
        m_artifactRepository = ar;
        m_wagonManager = wm;
        m_settings = settings;
        m_log = log;
        m_wagon = null;
    }
    public void disconnect(){
        if(m_wagon == null){
            m_log.error("must be connected first!");
            return;
        }
        try {
            m_wagon.disconnect();
        }
        catch ( ConnectionException e ) {
            m_log.error( "Error disconnecting wagon - ignored", e );
        }
    }
    
    public void connect() throws MojoExecutionException {
        String url = m_artifactRepository.getUrl();
        String id = m_artifactRepository.getId();
    
        Repository repository = new Repository( id, url );
    
        try {
            m_wagon = m_wagonManager.getWagon( repository );
            configureWagon( m_wagon, repository.getId() );
        }
        catch ( UnsupportedProtocolException e ) {
            throw new MojoExecutionException( "Unsupported protocol: '" + repository.getProtocol() + "'", e );
        }
        catch ( WagonConfigurationException e ) {
            throw new MojoExecutionException( "Unable to configure Wagon: '" + repository.getProtocol() + "'", e );
        }
     
    
        try {
            Debug debug = new Debug();
            m_wagon.addTransferListener( debug );
    
            ProxyInfo proxyInfo = getProxyInfo( m_settings );
            if ( proxyInfo != null ) {
                m_wagon.connect( repository, m_wagonManager.getAuthenticationInfo( id ), proxyInfo );
            }
            else {
                m_wagon.connect( repository, m_wagonManager.getAuthenticationInfo( id ) );
            }
    
        }
        catch ( ConnectionException e ) {
            throw new MojoExecutionException( "Error uploading file", e );
        }
        catch ( AuthenticationException e ) {
            throw new MojoExecutionException( "Error uploading file", e );
        }
    }
    
    public File get(String url) throws IOException, TransferFailedException, ResourceDoesNotExistException, AuthorizationException{
        
        if(m_wagon == null){
            m_log.error("must be connected first!");
            return null;
        }
        
        File file = File.createTempFile(String.valueOf(System.currentTimeMillis()), "tmp");
        m_wagon.get(url, file);
        return file;
    }

    public void put(File file, String url) throws TransferFailedException, ResourceDoesNotExistException, AuthorizationException {
        if(m_wagon == null){
            m_log.error("must be connected first!");
            return;
        }
        m_wagon.put(file, url);
    }
    
    public void changeArtifactRepository(ArtifactRepository ar) {
        m_log.error("no yet implemented");
    }
    
    
    
/**
 * Convenience method to map a <code>Proxy</code> object from the user system settings to a
 * <code>ProxyInfo</code> object.
 *
 * @return a proxyInfo object instancied or null if no active proxy is define in the settings.xml
 */
public static ProxyInfo getProxyInfo( Settings settings )
{
    ProxyInfo proxyInfo = null;
    if ( settings != null && settings.getActiveProxy() != null ) {
        Proxy settingsProxy = settings.getActiveProxy();

        proxyInfo = new ProxyInfo();
        proxyInfo.setHost( settingsProxy.getHost() );
        proxyInfo.setType( settingsProxy.getProtocol() );
        proxyInfo.setPort( settingsProxy.getPort() );
        proxyInfo.setNonProxyHosts( settingsProxy.getNonProxyHosts() );
        proxyInfo.setUserName( settingsProxy.getUsername() );
        proxyInfo.setPassword( settingsProxy.getPassword() );
    }

    return proxyInfo;
}

/**
 * Configure the Wagon with the information from serverConfigurationMap ( which comes from settings.xml )
 * 
 * @todo remove when {@link WagonManager#getWagon(Repository) is available}
 * @param wagon
 * @param repositoryId
 * @throws WagonConfigurationException
 */
private void configureWagon( Wagon wagon, String repositoryId )
    throws WagonConfigurationException
{
    if ( serverConfigurationMap.containsKey( repositoryId ) ) {
        ComponentConfigurator componentConfigurator = null;
        try {
            componentConfigurator = (ComponentConfigurator) container.lookup( ComponentConfigurator.ROLE );
            componentConfigurator.configureComponent( wagon, (PlexusConfiguration) serverConfigurationMap
                .get( repositoryId ), container.getContainerRealm() );
        }
        catch ( final ComponentLookupException e ) {
            throw new WagonConfigurationException( repositoryId, "Unable to lookup wagon configurator. Wagon configuration cannot be applied.", e );
        }
        catch ( ComponentConfigurationException e ) {
            throw new WagonConfigurationException( repositoryId, "Unable to apply wagon configuration.", e );
        }
        finally {
            if ( componentConfigurator != null ) {
                try {
                    container.release( componentConfigurator );
                }
                catch ( ComponentLifecycleException e ) {
                    m_log.error( "Problem releasing configurator - ignoring: " + e.getMessage() );
                }
            }

        }
    }
} 

public void contextualize( Context context )
    throws ContextException {
    container = (PlexusContainer) context.get( PlexusConstants.PLEXUS_KEY );
}
    
    
    
    
}
